'''
Calculate spatial weights for each of the 356 eddy covariance tower sites from
the FLUXNET2015/ La Thuille collection, for the purposes of fairly calibrating
against site data.

Sites that are within an EC tower footpring (< 2500 m) are assigned a weight
less than 1.0.
'''

import h5py
import numpy as np
from itertools import product
from mod17.utils import haversine

MOD16_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_tower_site_latent_heat_and_drivers_v4.h5'
MOD17_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v2.h5'

def main(hdf5):
    'Generates output CSV that knows the lng/lat coordinates of subgrid'
    with h5py.File(hdf5, 'r') as hdf:
        coords = hdf['coordinates/lng_lat'][:]

    weights = []
    # There are 2 coordinate axes, so dividing by 2 yields num. sites
    for i in range(coords.size // 2):
        p1 = coords[i]
        distances = np.array([
            haversine(p1, coords[j]) if i != j else np.nan
            for j in range(coords.size // 2)
        ]).round(1)
        # If any are found to be within 2500 m
        weights.append((distances < 2500).sum())

    weights = np.array(weights)
    weights += 1
    weights = np.ones(weights.shape) / weights
    with h5py.File(hdf5, 'a') as hdf:
        hdf.create_dataset('weights', weights.shape, np.float16, weights)


if __name__ == '__main__':
    main(MOD16_HDF)
