'''
Extracts statistics about the FLUXNET sites and writes out a CSV file.
'''

import csv
import os
import h5py
import numpy as np
from mod17 import MOD17, PFT_VALID
from mod17.utils import pft_dominant

MOD16_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_tower_site_latent_heat_and_drivers_v5.h5'
MOD17_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v3.h5'
OUTPUT_MOD17_CSV = '/home/arthur/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/docs/MOD17_FLUXNET_sites.csv'
PFT_CLASSES = (None, 'ENF', 'EBF', 'DNF', 'DBF', 'MF', 'CSH', 'OSH', 'WSV', 'SAV', 'GRS', None, 'CRO')

def main():
    with h5py.File(MOD17_HDF, 'r') as hdf:
        site_list = hdf['FLUXNET/site_id'][:].tolist()
        lng, lat = hdf['coordinates/lng_lat'][:].swapaxes(0,1).round(5).tolist()
        if hasattr(site_list[0], 'decode'):
            site_list = [s.decode('utf-8') for s in site_list]
        pft_map = pft_dominant(hdf['state/PFT'][:], site_list = site_list)
        tday = hdf['MERRA2/T10M_daytime'][:] - 273.15
        qv10m = hdf['MERRA2/QV10M_daytime'][:]
        ps = hdf['MERRA2/PS_daytime'][:]
        vpd = MOD17.vpd(qv10m, ps, tday)
    with open(OUTPUT_MOD17_CSV, 'w') as file:
        writer = csv.writer(file)
        writer.writerow((
            'site_id', 'longitude', 'latitude', 'code', 'PFT',
            'VPD_95th_ptile', 'VPD_99th_ptile'))
        for i in range(0, len(site_list)):
            q_vpd = np.nanpercentile(vpd[:,i], (95, 99)).astype(np.int32)
            if np.isnan(pft_map[i]):
                continue
            p = int(pft_map[i])
            writer.writerow((site_list[i], lng[i], lat[i], pft_map[i], PFT_CLASSES[p], *q_vpd))


if __name__ == '__main__':
    main()
