'''
DEPRECATED. If we were going to use climatology, we would pre-compute the
climatology first.

Extracts the MERRA-2 climatology at ORNL NPP sites:

    NPP/
      MERRA2_climatology/
        Tmin
        QV10M
        QV10M_daytime
        QV10M_nighttime
        SWGDN
        ...
        SWGNT
        ...
        LWGNT
        ...
'''

import glob
import os
import warnings
import netCDF4
import h5py
import numpy as np
from tqdm import tqdm
from pyl4c.lib.netcdf import netcdf_array
from pyl4c.spatial import xy_to_pixel

MOD17_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v5.h5'
MERRA2_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/MERRA2_climatology_1980-2000.h5'
MERRA2_GRID_SHAPE = (361, 576) # Something like 0.5 x 0.75 degree cells
MERRA2_SAMPLE = '/anx_lagr4/MODIS_VIIRS/calibration/inputs/MERRA-2/tavg1_2d_slv_Nx/MERRA2_200.tavg1_2d_slv_Nx.20000101.nc4.nc4'

def main():
    'Extracts the MERRA-2 climatology (365 days) at each ORNL NPP site'
    # Get coordinate arrays for each model
    with h5py.File(MOD17_HDF, 'r') as hdf:
        coords = hdf['NPP/lng_lat'][:]
    nc = netCDF4.Dataset(MERRA2_SAMPLE)
    # Get the GeoTransform for MERRA-2 arrays
    _, gt, wkt = netcdf_array(nc, ('T2M', 'lon', 'lat'))
    rows, cols = np.apply_along_axis(lambda a: xy_to_pixel(a, gt), 1, coords)\
        .swapaxes(0, 1)
    idx = np.ravel_multi_index((rows, cols), MERRA2_GRID_SHAPE)
    # Open file connections
    hdf = h5py.File(MERRA2_HDF, 'r')
    target_hdf = h5py.File(MOD17_HDF, 'a')
    if 'NPP/MERRA2_climatology' not in target_hdf.keys():
        target_hdf.create_group('NPP/MERRA2_climatology')
    for field in hdf.keys():
        print(f'Extracting {field}...')
        clim = hdf[field][:]
        subset = np.stack([clim[i].ravel()[idx] for i in range(0, 365)])
        target_hdf.create_dataset(
            f'NPP/MERRA2_climatology/{field}', subset.shape, np.float32, subset)
    target_hdf.flush()
    target_hdf.close()
    hdf.close()


if __name__ == '__main__':
    main()
