'''
Plots a global map of a field from the MERRA-2 climatology.

Turn the output images into an animated GIF with Image Magick:

    convert -delay 100 -loop 0 inputs*.png output.gif
'''

import os
import h5py
import numpy as np
from tqdm import tqdm
from matplotlib import pyplot

OUTPUT_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/MERRA2_climatology_1980-2000.h5'
OUTPUT_DIR = '/home/arthur/Downloads/plots'

def main():
    'Plots each day of each field'
    with h5py.File(OUTPUT_HDF, 'r') as hdf:
        for field in hdf.keys():
            if not os.path.exists(os.path.join(OUTPUT_DIR, field)):
                os.makedirs(os.path.join(OUTPUT_DIR, field))
            for day in tqdm(range(0, 365)):
                filename = os.path.join(OUTPUT_DIR, field, f'{field}_{day}.png')
                if os.path.exists(filename):
                    continue # Skip images already produced
                pyplot.imshow(hdf[field][day])
                pyplot.colorbar()
                pyplot.title(f'MERRA-2 "{field}" on Day {day+1}')
                pyplot.savefig(filename, dpi = 172)
                pyplot.close('all')


if __name__ == '__main__':
    main()
