'''
Plots the MODIS MOD15A2H fPAR and LAI data for:

- Each of the 356 eddy covariance sites from the FLUXNET2015/ La Thuille
    collection (used for calibrating MOD17);
- A subset of the FLUXNET2015 sites that have valid latent heat flux data
    (used for calibrating MOD16).
'''

import datetime
import os
import re
import h5py
import numpy as np
from tqdm import tqdm
from matplotlib import pyplot

MOD17_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v8.h5'
OUTPUT_DIR = '/home/arthur/Downloads/plots_MOD15'
N_SITES = 1569

def main():
    'Plots fPAR, LAI for each site'
    with h5py.File(MOD17_HDF, 'r') as hdf:
        for i in tqdm(range(N_SITES)):
            pyplot.figure(figsize = (8, 5))
            fpar = hdf['NPP/MOD15A2H_fPAR_clim_filt'][:,i].astype(np.float32)
            pyplot.plot(np.arange(1, 366), fpar, alpha = 0.4)
            pyplot.plot(np.arange(1, 366), np.nanmean(fpar, axis = -1), 'k-')
            pyplot.xlim((1, 365))
            pyplot.title('Site #%d' % i)
            pyplot.savefig(os.path.join(OUTPUT_DIR, '%d.png' % i))
            pyplot.close()


if __name__ == '__main__':
    main()
