# DEPRECATED: See 0902; Plotting the ORNL NPP calibration data by PFT

library(dplyr, warn.conflicts = F)
library(tidyr, warn.conflicts = F)
library(stringr)
library(hdf5r)
library(ggplot2)
library(latex2exp)
library(RColorBrewer)

PFT.TABLE <- data.frame(code = c(1:10, 12),
  abbrv = c('ENF', 'EBF', 'DNF', 'DBF', 'MF', 'CSH', 'OSH', 'WSV', 'SAV', 'GRS', 'CRO'))
PFT.TABLE$abbrv <- with(PFT.TABLE, ordered(abbrv, levels = abbrv))
COLOR.PALETTE <- c(brewer.pal(10, 'Paired')[1:4], brewer.pal(11, 'Paired')[11], brewer.pal(10, 'Paired')[5:10])


# Investigating the valid range of NPP #########################################

h5.file <- H5File$new('/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v8.h5', 'r')
gpp <- h5.file[['FLUXNET/GPP']][1:356,1:6575]
h5.file$close()

# Quantiles of mean annual GPP fluxes
q.gpp <- quantile(apply(gpp, 1, mean, na.rm = T) * 365, c(0.9, 0.95, 0.98, 0.99), na.rm = T)

# Quantiles of mean annual NPP fluxes assuming NPP:GPP ratio on [0.22, 0.79],
#   after Collalti & Prentice (2019)
q.gpp * 0.22
q.gpp * 0.79
# --> 98th percentile of NPP is then 2385
# --> 99th percentile of NPP is then 2628


# Plotting the calibration data ################################################

h5.file <- H5File$new('/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v8.h5', 'r')
npp <- h5.file[['NPP/NPP_total']][1:1646]
sites <- h5.file[['NPP/site_id']][1:1646]
pft <- h5.file[['NPP/PFT']][1:1646]
h5.file$close()
df.npp <- data.frame(NPP = npp, site.id = sites, code = pft) %>%
  left_join(PFT.TABLE, by = 'code') %>%
  rename(PFT = abbrv) %>%
  filter(!is.na(PFT))

tmp <- df.npp %>%
  filter(!is.na(NPP)) %>%
  filter(NPP <= 2385) %>%
  group_by(PFT) %>%
  summarize(Count = n())
ggplot(filter(df.npp, NPP <= 2385), mapping = aes(x = PFT, y = NPP / 1e3)) +
  geom_boxplot(size = 0.4) +
  annotate('text', label = tmp$Count, x = tmp$PFT, y = 2.8) +
  labs(x = NULL, y = TeX('Annual NPP ($kg\\,C\\,m^{-2}\\,yr^{-1}$)')) +
  scale_y_continuous(expand = c(0, 0), breaks = seq(0, 3, 1)) +
  coord_fixed(ylim = c(0, 3)) +
  theme_minimal()
ggsave(width = 5, height = 2.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202110/20211015_ORNL_NPP_by_PFT_with_cutoff-2385.png')

# Above-ground
h5.file <- H5File$new('/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v8.h5', 'r')
npp.above <- h5.file[['NPP/NPP_above']][1:1646]
h5.file$close()
df.npp <- data.frame(NPP = npp.above, site.id = sites, code = pft) %>%
  left_join(PFT.TABLE, by = 'code') %>%
  rename(PFT = abbrv) %>%
  filter(!is.na(PFT))

tmp <- df.npp %>%
  filter(!is.na(NPP)) %>%
  group_by(PFT) %>%
  summarize(Count = n())
ggplot(df.npp, mapping = aes(x = PFT, y = NPP / 1e3)) +
  geom_boxplot(size = 0.4) +
  annotate('text', label = tmp$Count, x = tmp$PFT, y = 4.95) +
  labs(x = NULL, y = TeX('Above-ground Annual NPP ($kg\\,C\\,m^{-2}\\,yr^{-1}$)')) +
  scale_y_continuous(expand = c(0, 0), breaks = seq(0, 4, 1)) +
  coord_fixed(ylim = c(0, 5.1)) +
  theme_minimal()
ggsave(width = 5, height = 2.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202110/20211015_ORNL_NPP-above_by_PFT.png')


# Below-ground
h5.file <- H5File$new('/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v8.h5', 'r')
npp.below <- h5.file[['NPP/NPP_below']][1:1646]
h5.file$close()
df.npp <- data.frame(NPP = npp.below, site.id = sites, code = pft) %>%
  left_join(PFT.TABLE, by = 'code') %>%
  rename(PFT = abbrv) %>%
  filter(!is.na(PFT))

tmp <- df.npp %>%
  filter(!is.na(NPP)) %>%
  group_by(PFT) %>%
  summarize(Count = n())
ggplot(df.npp, mapping = aes(x = PFT, y = NPP / 1e3)) +
  geom_boxplot(size = 0.4) +
  annotate('text', label = tmp$Count, x = tmp$PFT, y = 4.95) +
  labs(x = NULL, y = TeX('Below-ground Annual NPP ($kg\\,C\\,m^{-2}\\,yr^{-1}$)')) +
  scale_y_continuous(expand = c(0, 0), breaks = seq(0, 4, 1)) +
  coord_fixed(ylim = c(0, 5.1)) +
  theme_minimal()
ggsave(width = 5, height = 2.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202110/20211015_ORNL_NPP-below_by_PFT.png')


# Plotting the validation data #################################################

h5.file <- H5File$new('/anx_lagr4/MODIS_VIIRS/validation/ORNL_NPP_validation_data_v2.h5', 'r')
npp <- h5.file[['NPP/NPP_total']][1:2413]
sites <- h5.file[['NPP/site_id']][1:2413]
pft <- h5.file[['NPP/PFT']][1:2413]
h5.file$close()
df.npp <- data.frame(NPP = npp, site.id = sites, code = pft) %>%
  left_join(PFT.TABLE, by = 'code') %>%
  rename(PFT = abbrv) %>%
  filter(!is.na(PFT))

tmp <- df.npp %>%
  filter(!is.na(NPP)) %>%
  group_by(PFT) %>%
  summarize(Count = n())
ggplot(df.npp, mapping = aes(x = PFT, y = NPP / 1e3)) +
  geom_boxplot(size = 0.4) +
  annotate('text', label = tmp$Count, x = tmp$PFT, y = 4.95) +
  labs(x = NULL, y = TeX('Annual NPP ($kg\\,C\\,m^{-2}\\,yr^{-1}$)'), title = 'ORNL Validation Data') +
  scale_y_continuous(expand = c(0, 0), breaks = seq(0, 4, 1)) +
  coord_fixed(ylim = c(0, 5.1)) +
  theme_minimal()
ggsave(width = 5, height = 2.6, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202110/20211015_ORNL_NPP-validation_by_PFT.png')
