'''
Records the dominant PFT class for each FLUXNET site.
'''

import h5py
import numpy as np
from mod16.utils import pft_dominant as mod16_pft_dominant
from mod17.utils import pft_dominant as mod17_pft_dominant

MOD16_HDF5 = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_tower_site_latent_heat_and_drivers_v4.h5'
MOD17_HDF5 = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v2.h5'

def main():
    with h5py.File(MOD16_HDF5, 'a') as hdf:
        site_list = hdf['FLUXNET/site_id'][:].tolist()
        site_list = [s.decode('utf-8') for s in site_list]
        pft_map = hdf['state/PFT'][:]
        pft_dom = mod16_pft_dominant(pft_map, site_list)
        pft_dom[np.isnan(pft_dom)] = 0
        dset = hdf.create_dataset(
            'state/PFT_dominant', pft_dom.shape, np.int16, pft_dom)
        dset.attrs['description'] = 'PFT = 0 is used for invalid sites; i.e., not a real PFT'
    with h5py.File(MOD17_HDF5, 'a') as hdf:
        site_list = [
            s.decode('utf-8') for s in hdf['FLUXNET/site_id'][:].tolist()
        ]
        pft_map = hdf['state/PFT'][:]
        pft_dom = mod17_pft_dominant(pft_map, site_list)
        pft_dom[np.isnan(pft_dom)] = 0
        dset = hdf.create_dataset(
            'state/PFT_dominant', pft_dom.shape, np.int16, pft_dom)
        dset.attrs['description'] = 'PFT = 0 is used for invalid sites; i.e., not a real PFT'


if __name__ == '__main__':
    main()
