'''
Plots the MODIS MOD15A2H fPAR and LAI data for:

- Each of the 356 eddy covariance sites from the FLUXNET2015/ La Thuille
    collection (used for calibrating MOD17);
- A subset of the FLUXNET2015 sites that have valid latent heat flux data
    (used for calibrating MOD16).
'''

import datetime
import os
import re
import h5py
import numpy as np
from tqdm import tqdm
from matplotlib import pyplot
from mod17 import suppress_warnings

MOD16_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_tower_site_latent_heat_and_drivers_v4.h5'
MOD17_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_MOD17_tower_site_drivers_v8.h5'
OUTPUT_DIR = '/home/arthur/Downloads/plots_LH'

@suppress_warnings
def main(hdf_filename):
    'Plots fPAR, LAI for each site'
    with h5py.File(hdf_filename, 'r') as hdf:
        sites = hdf['FLUXNET/site_id'][:].tolist()
        dates = [
            datetime.date(*d) for d in hdf['time'][:].tolist()
        ]
        if hasattr(sites[0], 'decode'):
            sites = list(map(lambda s: s.decode('utf-8'), sites))
        for i, site_id in enumerate(tqdm(sites)):
            pyplot.figure(figsize = (8, 5))
            # fpar = hdf['MODIS/MOD15A2HGF_fPAR_interp'][:,i].astype(np.float32)
            fpar = hdf['FLUXNET/latent_heat'][:,i].astype(np.float32)
            pyplot.plot(dates, fpar)
            pyplot.xlim((dates[0], dates[-1]))
            pyplot.title(sites[i])
            pyplot.savefig(os.path.join(OUTPUT_DIR, '%s.png' % sites[i]))
            pyplot.close()


if __name__ == '__main__':
    main(MOD16_HDF)
