'''
Plots the MODIS MCD43GF gap-filled black-sky short-wave albedo at each of the
FLUXNET sites.
'''

import datetime
import os
import re
import h5py
import numpy as np
from tqdm import tqdm
from matplotlib import pyplot
from pyl4c import suppress_warnings

MOD16_HDF = '/anx_lagr4/MODIS_VIIRS/calibration/VIIRS_MOD16_tower_site_latent_heat_and_drivers_v4.h5'
OUTPUT_DIR = '/home/arthur/Downloads/plots'

@suppress_warnings
def main(hdf_filename):
    'Plots fPAR, LAI for each site'
    with h5py.File(hdf_filename, 'r') as hdf:
        sites = hdf['FLUXNET/site_id'][:].tolist()
        dates = [
            datetime.date(*d) for d in hdf['time'][:].tolist()
        ]
        if hasattr(sites[0], 'decode'):
            sites = list(map(lambda s: s.decode('utf-8'), sites))
        for i, site_id in enumerate(tqdm(sites)):
            pyplot.figure(figsize = (8, 5))
            albedo = hdf['MODIS/MCD43GF_black_sky_sw_albedo'][:,i].astype(np.float32)
            pyplot.plot(dates, np.nanmedian(albedo, axis = -1))
            pyplot.xlim((dates[0], dates[-1]))
            pyplot.title(sites[i])
            pyplot.savefig(os.path.join(OUTPUT_DIR, '%s.png' % sites[i]))
            pyplot.close()


if __name__ == '__main__':
    main(MOD16_HDF)
