# DEPRECATED. Process the TRY database export(s).
#   The approaches to constraining prior SLA did not work out.

# Could improve known Genera by using:
#   https://eol.org/pages/2895126
#   https://plants.usda.gov/home

# TODOs
# [x] The "mr_base" parameters are defined at 20 degrees C; are temperature
#   data available for the traits? Yes, but rarely.
# [x] Double-check Likelihood ratio test
# [x] Double-check `df.froot.leaf.ratio` for Leaf dry mass per plant UNITS

library(dplyr, warn.conflicts = F)
library(tidyr)
library(stringr)
library(ggplot2)
library(latex2exp)
library(bbmle, warn.conflicts = F)
options(stringsAsFactors = F)
Sys.setlocale("LC_ALL", "C")

CO2.MOLAR.MASS <- 44.009 # grams per mol
C.MOLAR.MASS <- 12.0107 # grams per mol

discrete.mode <- function (x) {
  u <- unique(x)
  tab <- tabulate(match(x, u))
  u[tab == max(tab)]
}

# Generally, ferns and herbacious perennials are not classified; also,
#   genera that are too diverse (or "comsopolitan") are not classified
GENERA.NOT.CLASSED <- c('Unidentified', 'Leaves', 'Stigmaphyllon', 'Saxifraga', 'Bonamia', 
  'Lupinus', 'Trichilia', 'Rinorea', 'Hironima', 'Santiria', 'Silene', 'Thelypteris',
  'Harrimanella', 'Potentilla', 'Antennaria', 'Ranunculus', 'Persicaria', 'Epilobium', 
  'Beckwithia', 'Trifolium', 'Galium', 'Rumex', 'Veronica', 'Achillea', 'Elaphoglossum',
  'Arabidopsis', 'Impatiens', 'Chenopodium', 'Astragalus', 'Sonchus', 'Aradopsis',
  'Capsella', 'Neslia', 'Petasites', 'Faramea', 'Viola', 'Centaurea', 'Erigeron',
  'Helianthemum', 'Taraxacum', 'Geranium', 'Solidago', 'Eriophorum', 'Symphyotrichum',
  'Lathyrus', 'Euphorbia', 'Campanula', 'Cirsium', 'Pilosella', 'Pedicularis', 
  'Atriplex', 'Lycium', 'Herniaria', 'Lespedeza', 'Sambucus', 'Mentha',
  'Lotus', 'Thalictrum', 'Hypericum', 'Anthurium', 'Senecio', 'Eupatorium', 
  'Pimpinella', 'Asclepias', 'Luzula', 'Sandwithia', 'Leucanthemum', 'Ambrosia',
  'Oenothera', 'Tabernaemontana', 'Polygonum', 'Erisma', 'Agathophora', 'Penstemon',
  'Linaria', 'Crepis', 'Urtica', 'Adenocalymma', 'Gentiana', 'Valeriana', 'Stellaria',
  'Anemone', 'Leontodon', 'Oxytropis', 'Myosotis', 'Anthriscus', 'Sedum',
  'Heisteria', 'Vochysia', 'Thymus', 'Christensonella', 'Cerastium', 'Filipendula',
  'Lysimachia', 'Laetia', 'Atractylis', 'Marila', 'Maianthemum', 'Eriogonum',
  'Erodium', 'Parthenocissus', 'Ulex', 'Geum', 'Equisetum', 'Hieracium', 'Iris',
  'Primula', 'Danaea', 'Limonium', 'Bellis', 'Wilkesia', 'Oxalis', 'Croton',
  'Asplenium', 'Sanguisorba', 'Alchemilla', 'Bidens', 'Verbascum', 'Smilax',
  'Lepidium', 'Cardamine', 'Amphicarpaea', 'Amphicarpa', 'Blechnum', 'Ambelania',
  'Prunella', 'Coreopsis', 'Thlaspi', 'Stachys', 'Dialium', 'Arenaria',
  'Tragopogon', 'Pallenis', 'Caesalpinia', 'Trophis', 'Peucedanum', 'Tanaecium',
  'Inula', 'Koelpinia', 'Campyloneurum', 'Rhinanthus', 'Hymenoxys', 'Goupia',
  'Spergularia', 'Heracleum', 'Andira', 'Acalypha', 'Sextonia', 'Notoceras',
  'Euclea', 'Dactylorhiza', 'Scorzoneroides', 'Convolvulus', 'Cassipourea',
  'Schefflera', 'Lamium', 'Fridericia', 'Simaba', 'Polygala', 'Parkia',
  'Duguetia', 'Bupleurum', 'Ouratea', 'Suaeda', 'Pseudopiptadenia',
  'Prinostemma', 'Clitoria', 'Alseis')

PFT.TABLE <- data.frame(code = c(1:10, 12),
  abbrv = c('ENF', 'EBF', 'DNF', 'DBF', 'MF', 'CSH', 'OSH', 'WSV', 'SAV', 'GRS', 'CRO'))
PFT.TABLE$abbrv <- with(PFT.TABLE, ordered(abbrv, levels = abbrv))

# Sources:
#   https://www.conifers.org/zz/gymnosperms.php
#   https://budburst.org/plants
#   https://www.tree-guide.com/
#   https://www.fao.org/fileadmin/templates/ess/documents/world_census_of_agriculture/appendix4_r7.pdf
#
# Genera with mostly "Semi-deciduous" species or that are not frequently
#   described as deciduous were classified as Evergreen
# Avicennia includes both shrubs and mangroves but only mangroves are present here
# GRS includes graminoids and sedges
PFT.pipeline <- function (df) {
  df %>%
  # Pseudolmedia is mis-spelled in some entries
  mutate(Genus = ifelse(Genus == 'Pseudomeldia', 'Pseudolmedia', Genus)) %>%
  mutate(PFT1 = case_when(
    Genus %in% c('Prunus', 'Luehea', 'Xylopia', 'Sterculia', 'Sclerolobium', 
        'Tachigali', 'Desmostachys', 'Coelocaryon', 'Desbordesia', 'Pouteria',
        'Protium', 'Eschweilera', 'Guatteria', 'Iryanthera', 'Eperua', 'Couepia',
        'Coffea', 'Casearia', 'Brosimum', 'Manilkara', 'Nectandra', 'Nothofagus',
        'Cupania', 'Gustavia', 'Zanthoxylum', 'Hirtella', 'Symphonia', 'Rhamnus',
        'Chrysochlamys', 'Tapirira', 'Scaphyglottis', 'Pogonophora', 'Vitex',
        'Pachira', 'Pseudolmedia', 'Handroanthus', 'Erythroxylum', 'Neea',
        'Poecilanthe', 'Poraqueiba', 'Symplocos', 'Conceveiba', 'Citrus',
        'Ceiba', 'Litsea', 'Pistacia', 'Platypodium', 'Chomelia'
      ) ~ 'Broadleaf',
    Genus %in% c('Abies', 'Acmopyle', 'Araucaria', 'Athrotaxis', 'Austrocedrus', 
        'Callitris', 'Calocedrus', 'Cathaya', 'Cedrus', 'Chamaecyparis', 
        'Cryptomeria', 'Cunninghamia', 'Cupressus', 'Cycas', 'Tsuga',
        'Dacrycarpus', 'Dacrydium', 'Diselma', 'Fitzroya', 'Widdringtonia',
        'Juniperus', 'Keteleeria', 'Lagarostrobos', 'Libocedrus', 'Manoao', 
        'Microbiota', 'Nothotsuga', 'Papuacedrus', 'Pherosphaera', 'Picea', 
        'Pinus', 'Platycladus', 'Prumnopitys', 'Pseudotsuga', 'Saxegothaea', 
        'Sciadopitys', 'Sequoia', 'Sequoiadendron', 'Taiwania', 'Taxus', 
        'Tetraclinis', 'Thuja', 'Thujopsis', 'Torreya'
      ) ~ 'ENF',
    Genus %in% c('Eucalyptus', 'Cassia', 'Stelechocarpus', 'Astronium', 'Orbigynia',
        'Ocotea', 'Bertholletia', 'Guarea', 'Cecropia', 'Naucleopsis', 'Sorocea',
        'Trymatococcus', 'Virola', 'Theobroma', 'Terminalia', 'Klainedoxa',
        'Musanga', 'Trema', 'Anacardium', 'Ficus', 'Chrysophyllum', 'Bowenia', 
        'Ceratozamia', 'Chigua', 'Dioon', 'Encephalartos', 'Lepidozamia', 
        'Macrozamia', 'Microcycas', 'Stangeria', 'Zamia', 'Ginkgo', 'Welwitschia',
        'Gnetum', 'Agathis', 'Wollemia', 'Afrocarpus', 'Nageia', 'Podocarpus',
        'Acacia', 'Avicennia', 'Pipturus', 'Pithecellobium', 'Inga', 'Swartzia',
        'Micropholis', 'Garcinia', 'Pourouma', 'Oxandra', 'Macaranga', 'Dicorynia',
        'Bocoa', 'Sloanea', 'Unonopsis', 'Tovomita', 'Quararibea', 'Oenocarpus',
        'Dendropanax', 'Tetragastris', 'Drypetes', 'Licaria', 'Talisia', 'Maytenus',
        'Chimarrhis', 'Lacmellea', 'Tapura', 'Mouriri', 'Vouacapoua', 'Perebea',
        'Castanopsis', 'Astrocaryum', 'Pradosia', 'Minquartia', 'Siparuna',
        'Couratari', 'Vismia', 'Dendrobangia', 'Copaifera', 'Thyrsodium',
        'Socratea', 'Simarouba', 'Dacryodes', 'Annona', 'Persea', 'Cinnamomum',
        'Beilschmiedia', 'Maquira', 'Cariniana', 'Rhodostemonodaphne', 'Lacistema',
        'Philodendron', 'Mytilaria', 'Gallesia'
      ) ~ 'EBF',
    Genus %in% c('Larix', 'Pseudolarix', 'Metasequoia', 'Glyptostrobus', 'Taxodium'
      ) ~ 'DNF',
    Genus %in% c('Acer', 'Quercus', 'Populus', 'Betula', 'Fagus', 'Juglans',
        'Carya', 'Cornus', 'Fraxinus', 'Liriodendron', 'Magnolia', 'Nyssa', 'Ulmus',
        'Oxydendron', 'Platanus', 'Robinia', 'Tilia', 'Distemonanthus', 'Sorbus',
        'Pterocarpus', 'Lophira', 'Triplochiton', 'Liquidambar', 'Castilla',
        'Spondias', 'Erythrophleum', 'Aesculus', 'Ailanthus', 'Alnus', 'Carapa',
        'Carpinus', 'Castanea', 'Catalpa', 'Celtis', 'Cercidiphyllum', 'Qualea',
        'Cladastris', 'Corylus', 'Gleditsia', 'Gymnocladus', 'Hemiptelea',
        'Maclura', 'Malus', 'Paulownia', 'Pterocarya', 'Staphylea', 'Dipterocarpus',
        'Styphnolobium', 'Lecythis', 'Ostrya', 'Aspidosperma', 'Jacaranda',
        'Laburnum'
      ) ~ 'DBF',
    Genus %in% c('Poa', 'Dactylis', 'Achnatherum', 'Andropogon', 'Bouteloua',
        'Bromus', 'Calamagrostis', 'Carex', 'Chasmanthium', 'Danthonia',
        'Deschampsia', 'Distichlis', 'Eragrostis', 'Hesperostipa', 'Hierochloe',
        'Juncus', 'Koeleria', 'Miscanthus', 'Nassella', 'Panicum', 'Pascopyrum',
        'Schizachyrium', 'Schoenoplectus', 'Scirpus', 'Spartina', 'Apera',
        'Sporobolus', 'Zizania', 'Stipa', 'Melilotus', 'Festuca', 'Elymus',
        'Anthoxanthum', 'Agrostis', 'Schismus', 'Phleum', 'Lolium', 'Arrhenatherum',
        'Helictotrichon', 'Alopecurus', 'Muhlenbergia', 'Holcus', 'Sorghastrum',
        'Dupontia', 'Cynosurus', 'Cleistogenes', 'Trisetum', 'Stipagrostis',
        'Arctophila', 'Aegilops', 'Leymus', 'Kobresia', 'Briza', 'Agropyron',
        'Chondrosum', 'Sesleria', 'Brachypodium', 'Aristida', 'Phragmites',
        'Glyceria', 'Eleocharis', 'Rytidosperma', 'Syzygium'
      ) ~ 'GRS',
    Genus %in% c('Cassiope', 'Diapensia', 'Empetrum', 'Salix', 'Rhododendron', 
        'Licania', 'Brenandendron', 'Zuelania', 'Combretum', 'Serjania',
        'Ephedra', 'Falcatifolium', 'Halocarpus', 'Lepidothamnus', 'Coussarea',
        'Microcachrys', 'Parasitaxus', 'Arctostaphylos', 'Argusia', 'Hedera',
        'Artemisia', 'Baccharis', 'Berberis', 'Byrsonima', 'Cercocarpus',
        'Coccoloba', 'Encelia', 'Euonymus', 'Dodonaea', 'Gaultheria', 'Dalbergia',
        'Heteromeles', 'Ilex', 'Kalmia', 'Morella', 'Myoporum', 'Argyroxiphium',
        'Osteomeles', 'Phoradendron', 'Rhus', 'Sida', 'Vaccinium', 'Psychotria',
        'Wikstroemia', 'Crataegus', 'Styrax', 'Calophyllum', 'Oxyria', 'Caragana',
        'Dubautia', 'Miconia', 'Viburnum', 'Rubus', 'Hebe', 'Cordia', 'Lonicera',
        'Eugenia', 'Rosa', 'Diospyros', 'Ardisia', 'Salsola', 'Sapium', 'Haloxylon',
        'Apeiba', 'Ribes', 'Lyonia', 'Psidium', 'Clusia', 'Buxus', 'Ormosia',
        'Myrcia', 'Elaeagnus', 'Arbutus', 'Urera', 'Buddleja', 'Albizia',
        'Gymnosporia', 'Daphne', 'Ligustrum', 'Calluna', 'Gaylussacia'
      ) ~ 'Shrub',
    Genus %in% c('Helianthus', 'Pisum', 'Solanum', 'Triticum', 'Vicia', 'Sorghum',
        'Medicago', 'Vitis', 'Oryza', 'Plantago', 'Brassica', 'Allium', 'Piper',
        'Fragaria', 'Eruca', 'Secale', 'Lens', 'Hordeum', 'Phaseolus', 'Fagopyrum',
        'Daucus', 'Manihot', 'Ceratonia', 'Ricinus', 'Apium', 'Capsicum', 'Cichorium',
        'Cicer', 'Zea', 'Vigna', 'Cucumis', 'Xanthasoma', 'Linum', 'Musa', 
        'Humulus', 'Lactuca', 'Pennisetum', 'Agave', 'Cucurbita', 'Glycine', 
        'Saccharum', 'Camellia'
      ) ~ 'CRO'
  )) %>%
  mutate(PFT2 = case_when( # NOTE: Order matters here! More specific terms should appear first
    str_detect(Habitat, 'evergreen lowland tropical forest') ~ 'EBF',
    str_detect(Habitat, 'pinelands') ~ 'ENF',
    str_detect(Habitat, 'meadow') ~ 'GRS',
    str_detect(Habitat, 'eucalypt') ~ 'EBF',
    str_detect(Habitat, 'larch') ~ 'DNF',
    str_detect(Habitat, 'agroforestry') ~ 'CRO',
    str_detect(Habitat, 'heath') ~ 'GRS',
    str_detect(Habitat, 'steppe') ~ 'GRS',
    str_detect(Habitat, 'savanna woodland') ~ 'SAV',
    str_detect(Habitat, 'savanna') ~ 'SAV',
    str_detect(Habitat, 'shrub') ~ 'Shrub',
    str_detect(Habitat, 'grassland') ~ 'GRS',
    str_detect(Habitat, 'pasture') ~ 'GRS',
    str_detect(Habitat, 'rainforest') ~ 'EBF',
    str_detect(Habitat, 'evergreen.*forest') ~ 'EBF',
    str_detect(Habitat, 'deciduous.*forest') ~ 'DBF',
    str_detect(Habitat, 'forest') ~ 'MF',
    str_detect(Habitat, 'scrub') ~ 'Shrub',
    str_detect(Habitat, 'woodlands') ~ 'MF',
    is.na(Habitat) ~ '')) %>%
  mutate(PFT3 = case_when(
    str_detect(VegType, 'Alpine tundra and steppe') ~ 'GRS',
    str_detect(VegType, 'Subtropical forest complex') ~ 'MF',
    str_detect(VegType, 'Boreal and subalpine forest and shrubland') ~ 'Shrub',
    str_detect(VegType, 'Subtropical deciduous and mixed forest') ~ 'MF',
    str_detect(VegType, 'Cold-resistant crops') ~ 'CRO',
    str_detect(VegType, 'Cold-resistant crops') ~ 'CRO',
    str_detect(VegType, 'Temperate desert') ~ 'OSH',
    str_detect(VegType, 'Temperate crops') ~ 'CRO',
    str_detect(VegType, 'Temperate steppe') ~ 'GRS',
    str_detect(VegType, 'Temperate deciduous forest complex') ~ 'DBF',
    str_detect(VegType, 'Cold-resistant crops with deciduous orchards') ~ 'CRO',
    str_detect(VegType, 'Temperate needleleaf forest') ~ 'ENF',
    str_detect(VegType, 'Tropical monsoon forest complex') ~ 'EBF',
    str_detect(VegType, 'Tropical rainforest complex') ~ 'EBF',
    str_detect(VegType, 'Temperate woodland and dry grassland') ~ 'WSV',
    is.na(VegType) ~ '')) %>%
  mutate(PFT1 = ifelse(is.na(PFT1), '', PFT1),
    PFT2 = ifelse(is.na(PFT2), '', PFT2),
    PFT3 = ifelse(is.na(PFT3), '', PFT3))
}

MCD12.pipeline <- function (df) {
  df %>%
    rename(OID = ObservationID) %>%
    mutate(ObservationID = str_split(OID, ',')) %>%
    unnest(ObservationID) %>%
    select(ObservationID, Longitude, Latitude, code = PFT_MCD12Q1) %>%
    mutate(ObservationID = as.numeric(ObservationID)) %>%
    left_join(PFT.TABLE, by = 'code') %>%
    mutate(abbrv = as.character(abbrv)) %>%
    rename(PFT.mapped = abbrv) %>%
    select(-code) %>%
    group_by(ObservationID, Latitude, Longitude) %>%
    summarize(PFT.mapped = first(PFT.mapped), .groups = 'drop') %>%
    ungroup() %>%
    select(-Latitude, -Longitude)
}

# For ancillary data:
# cat ~/Downloads/20988_07052022180913/20988.txt | awk -F "\t" '{ if ($11 == "TraitName" || $13 == "Climate_PFT_Biome equivalent_Poulter" || $13 == "Clustered vegetation type" || $13 == "Habitat / plot description" || $13 == "Latitude" || $13 == "Longitude") print }' > /anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA_attrs.txt

# Must use: Sys.setlocale("LC_ALL", "C")
df.attrs <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA_attrs.zip', 'TRY_SLA_attrs.txt'), 
    sep = '\t', fileEncoding = 'latin1') %>%
  select(ObservationID, DataName, OrigValueStr, StdValue) %>%
  group_by(ObservationID, DataName) %>%
  mutate(StdValue = as.character(StdValue)) %>%
  summarize(OrigValueStr = first(OrigValueStr),
    StdValue = first(StdValue)) %>%
  # Unify the different "value" sources, preferring StdValue
  mutate(value = ifelse(is.na(StdValue) | StdValue == "", OrigValueStr, StdValue)) %>%
  select(ObservationID, DataName, value) %>%
  spread(key = DataName, value = value) %>%
  rename(Biome = `Climate_PFT_Biome equivalent_Poulter`, 
    Habitat = `Habitat / plot description`,
    VegType = `Clustered vegetation type`) %>%
  mutate(Latitude = as.numeric(Latitude), 
    Longitude = as.numeric(Longitude),
    Habitat = str_to_lower(Habitat))


# Specific Leaf Area (SLA) ###########################

# For SLA dataset:
# cat ~/Downloads/20988_07052022180913/20988.txt | awk -F "\t" '{ if ($11 == "TraitName" || $11 == "Leaf area per leaf dry mass (specific leaf area, SLA or 1/LMA): petiole excluded") print }' > /anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA.txt

df.sla <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_SLA.zip', 'TRY_SLA.txt'), 
    sep = '\t', fileEncoding = 'latin1') %>%
  # For some reason, str_split won't work unless it is within groups
  group_by(AccSpeciesName) %>%
  mutate(Genus = str_to_title(str_split(AccSpeciesName, ' ', simplify = T)[1])) %>%
  ungroup() %>%
  left_join(df.attrs, by = 'ObservationID') %>%
  select(ObservationID, AccSpeciesName, TraitName, DataName, 
    OrigValueStr, OrigUnitStr, StdValue, UnitName, Biome, VegType, Habitat, 
    Latitude, Longitude, Reference, Genus) %>%
  PFT.pipeline %>%
  mutate(PFT = ifelse(PFT1 == '', 
    ifelse(PFT2 == '',
      ifelse(PFT3 == '', NA, PFT3), PFT2), PFT1)) %>%
  select(-PFT1, -PFT2, -PFT3)

df.sla.pft <- read.csv('~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/TRY_SLA_PFT_from_MCD12Q1.csv') %>%
  MCD12.pipeline

# Join the MODIS MCD12Q1 PFT data for those Genera not already assigned PFTs
df.sla.joined <- df.sla %>%
  filter(!is.na(StdValue)) %>%
  left_join(df.sla.pft, by = 'ObservationID') %>%
  mutate(PFT = ifelse(is.na(PFT), PFT.mapped, PFT)) %>%
  mutate(StdValue = ifelse(TraitName != 'Plant biomass and allometry: Leaf dry mass per plant', StdValue,
  ifelse(OrigUnitStr %in% c('g', 'g/plant'), as.numeric(OrigValueStr),
    ifelse(OrigUnitStr == 'kg plant-1', as.numeric(OrigValueStr) * 1e3,
      ifelse(OrigUnitStr == 'mg', as.numeric(OrigValueStr) / 1e3, NA)))))

# Consolidate functional groups into PFTs, then aggregate traits by species, taking
#   the majority PFT class
df.sla.by.species <- bind_rows(
  filter(df.sla.joined, PFT %in% c('ENF', 'EBF', 'DNF', 'DBF', 'GRS', 'CRO')),
  mutate(filter(df.sla.joined, PFT %in% c('ENF', 'EBF', 'DNF', 'DBF', 'Broadleaf')),
    PFT = 'MF'),
  mutate(filter(df.sla.joined, PFT %in% c('Shrub', 'CSH')), PFT = 'CSH'),
  mutate(filter(df.sla.joined, PFT %in% c('Shrub', 'OSH')), PFT = 'OSH'),
  mutate(df.sla.joined, PFT = 'WSV'),
  mutate(df.sla.joined, PFT = 'SAV')) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
  # Remove varieties and trailing whitespace
  mutate(AccSpeciesName = str_remove(str_remove(AccSpeciesName, 'var\\..*'), ' $')) %>%
  group_by(AccSpeciesName, Genus, TraitName) %>%
  summarize(StdValue = median(StdValue, na.rm = T), 
    PFT = discrete.mode(PFT))

df.sla.by.species %>%
  filter(TraitName == "Leaf carbon (C) content per leaf dry mass") %>%
  group_by(PFT) %>%
ggplot(mapping = aes(x = PFT, y = StdValue)) +
  geom_boxplot() +
  coord_flip() +
  labs(y = latex2exp::TeX('Leaf C Content $(mg\\,C\\,g^{-1})$')) +
  theme_linedraw()
ggsave(width = 4.5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_LeafC.png')

df.sla.by.species %>%
  filter(TraitName == "Plant biomass and allometry: Leaf dry mass per plant") %>%
  group_by(PFT) %>%
ggplot(mapping = aes(x = PFT, y = StdValue)) +
  geom_boxplot() +
  coord_flip() +
  scale_y_log10() +
  labs(y = latex2exp::TeX('Leaf dry mass (g)')) +
  theme_linedraw()
ggsave(width = 4.5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_LeafMass.png')

# Match SLA and Leaf carbon content by Species name, then aggregate by species,
#   taking the most common PFT
df.sla.summary <- df.sla.by.species %>% 
  select(AccSpeciesName, TraitName, StdValue, Genus, PFT) %>% 
  mutate(TraitName = str_extract(TraitName, '(SLA|Leaf carbon|Leaf dry mass)')) %>%
  filter(!(TraitName == 'SLA' & StdValue > 200)) %>%
  ungroup()

df.sla.summary %>%
  filter(TraitName == 'Leaf carbon') %>%
  select(AccSpeciesName, Genus, PFT, LeafC = StdValue) %>%
  left_join(select(filter(df.sla.summary, TraitName == 'SLA'), -TraitName),
    by = c('AccSpeciesName', 'Genus', 'PFT')) %>%
  rename(LMA.inv = StdValue) %>%
  # Convert (1/LMA) from (mm2 mg-1) to (m2 kg-1)
  mutate(LMA.inv = LMA.inv * 1e6) %>%
  # Convert (mg g-1) to (g kg-1) %>%
  mutate(LeafC = LeafC * 1e3) %>%
  # SLA in MOD17 is denominated by Carbon mass, not total dry mass, so divide
  #   (1/LMA) by the Leaf C per unit dry weight to obtain in terms of per C
  mutate(SLA = LMA.inv / LeafC) %>%
  # HOWEVER, this is units of (m2 g-1) when we need (m2 kg-1)
  mutate(SLA = SLA * (1/1e3)) %>%
  filter(!is.na(SLA)) %>%
  summary


# Specific Leaf Area (SLA) as LAI per C ########################

# cat ~/Downloads/21115_20052022145340/21115.txt | awk -F "\t" '{ if ($11 == "TraitName" || $11 == "Leaf area index (LAI) of a single plant") print '} > TRY_LAI.txt

df.lai <- read.csv(unz('/anx_lagr4/MODIS_VIIRS/calibration/data/TRY/TRY_LAI.zip', 'TRY_LAI.txt'), 
    sep = '\t', fileEncoding = 'latin1') %>%
  # For some reason, str_split won't work unless it is within groups
  group_by(AccSpeciesName) %>%
  mutate(Genus = str_to_title(str_split(AccSpeciesName, ' ', simplify = T)[1])) %>%
  ungroup() %>%
  left_join(df.attrs, by = 'ObservationID') %>%
  select(ObservationID, AccSpeciesName, TraitName, DataName, 
    OrigValueStr, OrigUnitStr, StdValue, UnitName, Biome, VegType, Habitat, 
    Latitude, Longitude, Reference, Genus) %>%
  PFT.pipeline %>%
  mutate(PFT = ifelse(PFT1 == '', 
    ifelse(PFT2 == '',
      ifelse(PFT3 == '', NA, PFT3), PFT2), PFT1)) %>%
  select(-PFT1, -PFT2, -PFT3)

df.lai.pft <- read.csv('~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/data/TRY/TRY_LAI_PFT_from_MCD12Q1.csv') %>%
  MCD12.pipeline

# Join the MODIS MCD12Q1 PFT data for those Genera not already assigned PFTs
df.lai.joined <- df.lai %>%
  # NOTE: LAI has not StdValue but all the upstream unit names are different
  #   names for the same thing
  mutate(StdValue = as.numeric(OrigValueStr)) %>%
  filter(!is.na(StdValue)) %>%
  left_join(df.lai.pft, by = 'ObservationID') %>%
  mutate(PFT = ifelse(is.na(PFT), PFT.mapped, PFT))

# Consolidate functional groups into PFTs, then aggregate traits by species, taking
#   the majority PFT class
df.lai.by.species <- bind_rows(
  filter(df.lai.joined, PFT %in% c('ENF', 'EBF', 'DNF', 'DBF', 'GRS', 'CRO')),
  mutate(filter(df.lai.joined, PFT %in% c('ENF', 'EBF', 'DNF', 'DBF', 'Broadleaf')),
    PFT = 'MF'),
  mutate(filter(df.lai.joined, PFT %in% c('Shrub', 'CSH')), PFT = 'CSH'),
  mutate(filter(df.lai.joined, PFT %in% c('Shrub', 'OSH')), PFT = 'OSH'),
  mutate(df.lai.joined, PFT = 'WSV'),
  mutate(df.lai.joined, PFT = 'SAV')) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
  # Remove varieties and trailing whitespace
  mutate(AccSpeciesName = str_remove(str_remove(AccSpeciesName, 'var\\..*'), ' $')) %>%
  group_by(AccSpeciesName, Genus, TraitName) %>%
  summarize(StdValue = median(StdValue, na.rm = T), 
    PFT = discrete.mode(PFT))

df.sla.joined2 <- df.sla.joined %>%
  filter(TraitName %in% c('Leaf carbon (C) content per leaf dry mass', 
    'Plant biomass and allometry: Leaf dry mass per plant')) %>%
  mutate(TraitName = str_extract(TraitName, 'Leaf carbon|Plant biomass')) %>%
  mutate(TraitName = ifelse(TraitName == 'Plant biomass', 'Leaf dry mass', TraitName)) %>%
  mutate(StdValue = ifelse(TraitName == 'Leaf carbon', StdValue, 
    ifelse(OrigUnitStr %in% c('g', 'g/plant'), as.numeric(OrigValueStr),
      ifelse(OrigUnitStr == 'kg plant-1', 1e3 * as.numeric(OrigValueStr),
        ifelse(OrigUnitStr == 'mg', as.numeric(OrigValueStr) / 1e3, NA)))))
  
df.sla.by.species2 <- bind_rows(
  filter(df.sla.joined2, PFT %in% c('ENF', 'EBF', 'DNF', 'DBF', 'GRS', 'CRO')),
  mutate(filter(df.sla.joined2, PFT %in% c('ENF', 'EBF', 'DNF', 'DBF', 'Broadleaf')),
    PFT = 'MF'),
  mutate(filter(df.sla.joined2, PFT %in% c('Shrub', 'CSH')), 
    PFT = 'CSH'),
  mutate(filter(df.sla.joined2, PFT %in% c('Shrub', 'OSH')), 
    PFT = 'OSH'),
  mutate(df.sla.joined2, PFT = 'WSV'),
  mutate(df.sla.joined2, PFT = 'SAV')) %>%
  mutate(PFT = ordered(PFT, levels = PFT.TABLE$abbrv)) %>%
  # Remove varieties and trailing whitespace
  mutate(AccSpeciesName = str_remove(str_remove(AccSpeciesName, 'var\\..*'), ' $')) %>%
  group_by(AccSpeciesName, Genus, TraitName) %>%
  summarize(StdValue = median(StdValue, na.rm = T), 
    PFT = discrete.mode(PFT))

# NOTE: Some PFTs have no species observations, so we'll use the entire
#   sample for each PFT; GRS and CRO have too few species to differentiate
# with(df.lai.by.species, table(PFT))

df.sla.by.species2 %>%
  filter(TraitName == 'Leaf dry mass') %>%
ggplot(mapping = aes(x = PFT, y = StdValue)) +
  geom_boxplot() +
  coord_flip() +
  labs(y = latex2exp::TeX('Leaf Dry Mass per Plant (g)')) +
  theme_linedraw()
ggsave(width = 4.5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_LeafDryMass.png')

df.lai.by.species %>%
ggplot(mapping = aes(x = PFT, y = StdValue)) +
  geom_boxplot() +
  scale_y_log10() +
  coord_flip() +
  labs(y = 'LAI') +
  theme_linedraw()
ggsave(width = 4.5, height = 4.5, dpi = 172,
  file = '~/Workspace/NTSG/projects/Y2021_MODIS-VIIRS/outputs/202205/20220505_TRY_LAI.png')

# df.sla2
bind_rows(df.lai.by.species, df.sla.by.species2) %>%
  spread(key = TraitName, value = StdValue) %>%
  mutate(LeafC.kg = `Leaf carbon` * `Leaf dry mass` / 1e6) %>%
  mutate(SLA = `Leaf area index (LAI) of a single plant` / LeafC.kg) %>%
  summary

# NO: These values are far too large


